/*********************************************************************
 *
 * Copyright 2003 NetBurner  ALL RIGHTS RESERVED
 *   Permission is hereby granted to purchasers of NetBurner Hardware
 *   to use or modify this computer program for any use as long as the
 *   resultant program is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or it's derivatives in part or
 *   in whole are granted.
 *
 *   It may be possible to license this or other NetBurner software for
 *   use on non NetBurner Hardware. Please contact Licensing@Netburner.com
 *   for more information.
 *
 *   NetBurner makes no representation or warranties with respect to 
 *   the performance of this computer program, and specifically disclaims
 *   any responsibility for any damages, special or consequential, 
 *   connected with the use of this program.
 *
 * Information is available at:  http://www.netburner.com
 * E-Mail info@netburner.com
 *
 * Support is availible at: support@netburner.com
 *
 *********************************************************************/

#include "predef.h"
#include <stdio.h>
#include <stdlib.h>
#include <startnet.h>
#include <iosys.h>
#include <utils.h>
#include <ip.h>
#include <tcp.h>
#include <htmlfiles.h>
#include <http.h>
#include <string.h>
#include <basictypes.h>
#include "iadweb.h"


#define VERIFY_KEY (0x10220002) 	// NV Settings key code


typedef struct NV_SettingsStruct 	// non-volatile storage structure
{
	DWORD 	VerifyKey;
	BOOL 	bUTC;
	int		nLocalOffset;
	BOOL  	bCkboxLeapSecDay;
	int		nLeapSecDay;
	BOOL	bLeapSecDayAdd;
	BOOL	bEnableDST;
};


//----- global vars -----
char	gPostBuf[1000];
NV_SettingsStruct NV_Settings;
const char FirmwareVersion[] = "1.0, 12/17/2002";



/*-------------------------------------------------------------------
 Check NV Settings
 Assign default values if VerifyKey is not valid, otherwise load
 stored flash values.
 ------------------------------------------------------------------*/
void CheckNVSettings()
{
	NV_SettingsStruct *pData 	= (NV_SettingsStruct *)GetUserParameters();
	NV_Settings.VerifyKey   	= pData->VerifyKey;
	NV_Settings.bUTC			= pData->bUTC;
	NV_Settings.nLocalOffset	= pData->nLocalOffset;
	NV_Settings.bCkboxLeapSecDay	= pData->bCkboxLeapSecDay;
	NV_Settings.nLeapSecDay		= pData->nLeapSecDay;
	NV_Settings.bLeapSecDayAdd	= pData->bLeapSecDayAdd;
	NV_Settings.bEnableDST		= pData->bEnableDST;

	if (NV_Settings.VerifyKey != VERIFY_KEY)
	{
        iprintf("Reset NV Memory Defaults\n");
        
        NV_Settings.VerifyKey   	= VERIFY_KEY;
        NV_Settings.bUTC	   		= TRUE;
        NV_Settings.nLocalOffset	= 0;
        NV_Settings.bCkboxLeapSecDay	= FALSE;
        NV_Settings.nLeapSecDay		= 0;
        NV_Settings.bLeapSecDayAdd	= TRUE;
        NV_Settings.bEnableDST		= TRUE;
        
        SaveUserParameters(&NV_Settings, sizeof(NV_Settings));
   }
}



/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebUTC(int sock, PCSTR url)
{
	if (NV_Settings.bUTC)
	   writestring(sock, "CHECKED=\"1\" ");
}

/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebLocal(int sock, PCSTR url)
{
	if (! NV_Settings.bUTC)
	   writestring(sock, "CHECKED=\"1\" ");
}


/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebLocalOffset(int sock, PCSTR url)
{
	char buf[80];

	sprintf(buf, "VALUE=\"%d\" ", NV_Settings.nLocalOffset);
	writestring(sock, buf);   
}



/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebLeapCkbox(int sock, PCSTR url)
{
	if (NV_Settings.bCkboxLeapSecDay)
   	writestring(sock, "checked");
}

/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebLeapSecDay(int sock, PCSTR url)
{
	char buf[80];

	sprintf(buf, "VALUE=\"%d\" ", NV_Settings.nLeapSecDay);
	writestring(sock, buf);   
}

/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebLeapSecDayAdd(int sock, PCSTR url)
{
	if (NV_Settings.bLeapSecDayAdd)
	   writestring(sock, "CHECKED=\"1\" ");
}

/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebLeapSecDaySub(int sock, PCSTR url)
{
	if (! NV_Settings.bLeapSecDayAdd)
	   writestring(sock, "CHECKED=\"1\" ");
}

/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebEnableDST(int sock, PCSTR url)
{
	if (NV_Settings.bEnableDST)
	   writestring(sock, "CHECKED=\"1\" ");
}

/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebDisableDST(int sock, PCSTR url)
{
	if (! NV_Settings.bEnableDST)
	   writestring(sock, "CHECKED=\"1\" ");
}


/*-------------------------------------------------------------------
 Handle HTTP Post

 WARNING WARNING WARNING     CAUTION CAUTION CAUTION
 The User data space is 8K in size.  The object we are storing is
 less than 200 bytes long, thus it is probably ok to make an automatic
 variable out of it.  It would NOT BE OK to make an 8K automatic
 variable.  The coices for doing this with an 8K object...

 First choice: make a global variable, this way the linker will 
 allocate space for it. All errors will be at link time not run time.

 Second choice: Increase the HTTP stack size in constants.h and
 recompile the whole system directory.

 Third choice: use malloc and free.  The only problem is what do 
 you do if malloc fails?
 ------------------------------------------------------------------*/
int MyDoPost(int sock, char * url, char * pData, char * rxBuffer)
{
	int max_chars = 40;
	char *buf = gPostBuf;	// post buffer is global

	iprintf(pData);	// print all data sent from app


	// Process UTC/Local radio buttons
	if (ExtractPostData("radioUTC-Local", pData, buf, max_chars) == -1)
		iprintf("Error reading post data: radioUTC\r\n");
	else
	{
		if (strcmp(buf, "UTC") == 0)
			NV_Settings.bUTC = TRUE;
		else
			NV_Settings.bUTC = FALSE;
	}

	// Process Local Offset Hours text field
	if (ExtractPostData("tfLocalOffHours", pData, buf, max_chars) == -1)
		iprintf( "Error reading post data: tfLocalOffHours\r\n");
	else
		NV_Settings.nLocalOffset = atoi(buf);
	
	// Process Leap Second Day checkbox
	if (ExtractPostData("ckboxLeapSecDay", pData, buf, max_chars) == -1)
		NV_Settings.bCkboxLeapSecDay = FALSE;
	else
		NV_Settings.bCkboxLeapSecDay = TRUE;
	
	// Process Leap Second Day text field
	if (ExtractPostData("tfLeapSecDay", pData, buf, max_chars) == -1)
		iprintf( "Error reading post data: tfLeapSecDay\r\n");
	else
		NV_Settings.nLeapSecDay = atoi(buf);
	
	// Process Leap Sec Day Add/Sub radio buttons
	if (ExtractPostData("radioLeapAddSub", pData, buf, max_chars) == -1)
		iprintf("Error reading post data: radioLeapAddSub\r\n");
	else
	{
		if (strcmp(buf, "Add") == 0)
			NV_Settings.bLeapSecDayAdd = TRUE;
		  else
			NV_Settings.bLeapSecDayAdd = FALSE;
	}

	// Process DST radio buttons
	if (ExtractPostData("radioDST", pData, buf, max_chars) == -1)
		iprintf("Error reading post data: radioDST\r\n");
   else
   {
   	if (strcmp(buf, "Enable") == 0)
      	NV_Settings.bEnableDST = TRUE;
      else
      	NV_Settings.bEnableDST = FALSE;
   }


	// Now to store it in flash.  
   // WARNING: If new settings are added, remember to add them to
   //          NV Settings default initalization!
   NV_Settings.VerifyKey = VERIFY_KEY;
	if ( SaveUserParameters(&NV_Settings, sizeof(NV_Settings)) != 0 )
  		iprintf( "New Settings Saved\r\n");
   else
      iprintf( "ERROR: Could not save new settings\r\n");

   
	// We have to respond to the post with a new HTML page. In
   // this case we will redirect so the browser will go to 
   // that URL for the response.
	RedirectResponse(sock, "index.htm");

	return 0;
}


/*-------------------------------------------------------------------
 Register Post
 This function sets the HTTP POST handler to point to our
 function.
 ------------------------------------------------------------------*/
void RegisterPost()
{
	SetNewPostHandler(MyDoPost);
}

